@@ -53,86 +53,82 @@ int tests_failed = 0;
     } while (0)
 
 /* ========================================================================= */
-/* Helper Functions for Creating search_params_t    */
+/* Common helper for both literal and regex params                          */
 /* ========================================================================= */
-
-// Helper to create params for single literal pattern
-search_params_t create_literal_params(const char *pattern, bool case_sensitive, bool count_lines, bool only_match)
+static search_params_t create_base_params(const char *pattern,
+                                          bool case_sensitive,
+                                          bool count_lines,
+                                          bool only_match,
+                                          bool use_regex)
 {
     search_params_t params = {0};
     params.patterns = malloc(sizeof(char *));
     params.pattern_lens = malloc(sizeof(size_t));
     if (!params.patterns || !params.pattern_lens)
     {
-        perror("Failed to allocate memory for single pattern params");
+        perror("Failed to allocate memory for params");
         exit(EXIT_FAILURE);
     }
-    params.patterns[0] = (char *)pattern; // Cast needed as patterns is const char**
+    params.patterns[0] = (char *)pattern;
     params.pattern_lens[0] = strlen(pattern);
     params.num_patterns = 1;
     params.case_sensitive = case_sensitive;
-    params.use_regex = false;
-    params.count_lines_mode = count_lines && !only_match;
-    params.count_matches_mode = count_lines && only_match; // Not directly used, but set for clarity
-    params.track_positions = !(count_lines && !only_match);
-    params.compiled_regex = NULL;
-    params.max_count = SIZE_MAX; // Default: no limit
-
-    // Set legacy fields for compatibility with older test functions if needed
-    params.pattern = params.patterns[0];
-    params.pattern_len = params.pattern_lens[0];
-
-    return params;
-}
-
-// Helper to create params for single regex pattern
-search_params_t create_regex_params(const char *pattern, bool case_sensitive, bool count_lines, bool only_match)
-{
-    search_params_t params = {0};
-    params.patterns = malloc(sizeof(char *));
-    params.pattern_lens = malloc(sizeof(size_t));
-    if (!params.patterns || !params.pattern_lens)
-    {
-        perror("Failed to allocate memory for single regex params");
-        exit(EXIT_FAILURE);
-    }
-    params.patterns[0] = (char *)pattern;     // Cast needed
-    params.pattern_lens[0] = strlen(pattern); // Store length even for regex
-    params.num_patterns = 1;
-    params.case_sensitive = case_sensitive;
-    params.use_regex = true;
+    params.use_regex = use_regex;
     params.count_lines_mode = count_lines && !only_match;
     params.count_matches_mode = count_lines && only_match;
     params.track_positions = !(count_lines && !only_match);
-    params.max_count = SIZE_MAX; // Default: no limit
-
-    // Compile the regex (allocate locally for the test)
-    regex_t *compiled_regex = malloc(sizeof(regex_t));
-    if (!compiled_regex)
-    {
-        perror("Failed to allocate memory for compiled regex");
-        exit(EXIT_FAILURE);
-    }
-    int rflags = REG_EXTENDED | REG_NEWLINE | (case_sensitive ? 0 : REG_ICASE);
-    int ret = regcomp(compiled_regex, pattern, rflags);
-    if (ret != 0)
-    {
-        char ebuf[256];
-        regerror(ret, compiled_regex, ebuf, sizeof(ebuf));
-        fprintf(stderr, "Regex compilation error in test setup: %s\n", ebuf);
-        free(compiled_regex);
-        exit(EXIT_FAILURE);
-    }
-    params.compiled_regex = compiled_regex; // Assign the compiled regex
-
-    // Set legacy fields if needed (less relevant for regex)
+    params.compiled_regex = NULL;
+    params.max_count = SIZE_MAX;
+    params.ac_trie = NULL; // Initialize to NULL
+    // legacy single‐pattern fields
     params.pattern = params.patterns[0];
     params.pattern_len = params.pattern_lens[0];
-
     return params;
 }
 
-// Cleanup function for params (frees regex if allocated)
+search_params_t create_literal_params(const char *pattern,
+                                      bool case_sensitive,
+                                      bool count_lines,
+                                      bool only_match)
+{
+    return create_base_params(pattern,
+                              case_sensitive,
+                              count_lines,
+                              only_match,
+                              false);
+}
+
+search_params_t create_regex_params(const char *pattern,
+                                    bool case_sensitive,
+                                    bool count_lines,
+                                    bool only_match)
+{
+    search_params_t params = create_base_params(pattern,
+                                                case_sensitive,
+                                                count_lines,
+                                                only_match,
+                                                true);
+    regex_t *compiled = malloc(sizeof(regex_t));
+    if (!compiled)
+    {
+        perror("Failed to allocate memory for regex_t");
+        exit(EXIT_FAILURE);
+    }
+    int rflags = REG_EXTENDED | REG_NEWLINE | (case_sensitive ? 0 : REG_ICASE);
+    int ret = regcomp(compiled, pattern, rflags);
+    if (ret != 0)
+    {
+        char ebuf[256];
+        regerror(ret, compiled, ebuf, sizeof(ebuf));
+        fprintf(stderr, "Regex compilation error in test setup: %s\n", ebuf);
+        free(compiled);
+        exit(EXIT_FAILURE);
+    }
+    params.compiled_regex = compiled;
+    return params;
+}
+
+/* Cleanup function for params (frees regex if allocated) */
 void cleanup_params(search_params_t *params)
 {
     if (params->use_regex && params->compiled_regex)
@@ -152,6 +148,12 @@ void cleanup_params(search_params_t *params)
         free(params->pattern_lens);
         params->pattern_lens = NULL;
     }
+    // Free Aho-Corasick trie if allocated
+    if (params->ac_trie)
+    {
+        ac_trie_free(params->ac_trie);
+        params->ac_trie = NULL;
+    }
     // Note: Does not free ac_trie, handled separately
 }
 
@@ -815,7 +817,7 @@ void test_max_count_new(void)
 
     // --- Aho-Corasick Search (Multiple Patterns) ---
     printf("--- Aho-Corasick Search ---\n");
-    const char *ac_text = "apple banana apple orange apple grape apple";
+    const char *ac_text = "apple banana apple orange apple banana orange apple orange";
     size_t ac_text_len = strlen(ac_text);
     const char *ac_patterns[] = {"apple", "orange"};
     size_t ac_pattern_lens[] = {5, 6};
@@ -829,31 +831,50 @@ void test_max_count_new(void)
         .count_lines_mode = false,
         .count_matches_mode = false,
         .compiled_regex = NULL,
-        .max_count = SIZE_MAX // Default
+        .max_count = SIZE_MAX, // Default
+        .ac_trie = NULL        // Initialize to NULL
     };
 
+    // Build trie before using it
+    params_ac.ac_trie = ac_trie_build(&params_ac);
+    if (!params_ac.ac_trie)
+    {
+        printf("✗ FAIL: Failed to build Aho-Corasick trie in max_count test\n");
+        tests_failed++;
+        return; // Cannot proceed
+    }
+
+    // Test with max_count = 3
     params_ac.max_count = 3;
     result = match_result_init(10);
-    TEST_ASSERT(aho_corasick_search(&params_ac, ac_text, ac_text_len, result) == 3, "Aho-Corasick finds 3 matches with limit 3");
+    TEST_ASSERT(aho_corasick_search(&params_ac, ac_text, ac_text_len, result) == 3,
+                "Aho-Corasick finds 3 matches with limit 3");
     TEST_ASSERT(result->count == 3, "Aho-Corasick result has 3 positions with limit 3");
     match_result_free(result);
     result = NULL;
 
-    params_ac.max_count = 5; // Total matches are 4 'apple' + 1 'orange' = 5
+    // Test with max_count = 5
+    params_ac.max_count = 5; // There are 7 total matches (4 apple, 3 orange)
     result = match_result_init(10);
-    TEST_ASSERT(aho_corasick_search(&params_ac, ac_text, ac_text_len, result) == 5, "Aho-Corasick finds 5 matches with limit 5");
+    TEST_ASSERT(aho_corasick_search(&params_ac, ac_text, ac_text_len, result) == 5,
+                "Aho-Corasick finds 5 matches with limit 5");
     TEST_ASSERT(result->count == 5, "Aho-Corasick result has 5 positions with limit 5");
     match_result_free(result);
     result = NULL;
 
+    // Test with max_count = 6 (should find 6 matches)
     params_ac.max_count = 6;
     result = match_result_init(10);
-    TEST_ASSERT(aho_corasick_search(&params_ac, ac_text, ac_text_len, result) == 5, "Aho-Corasick finds 5 matches with limit 6");
-    TEST_ASSERT(result->count == 5, "Aho-Corasick result has 5 positions with limit 6");
+    uint64_t matches_with_limit_6 = aho_corasick_search(&params_ac, ac_text, ac_text_len, result);
+    TEST_ASSERT(matches_with_limit_6 == 6, // Expect 6 matches
+                "Aho-Corasick finds 6 matches with limit 6");
+    TEST_ASSERT(result->count == 6, // Expect 6 positions
+                "Aho-Corasick result has 6 positions with limit 6");
     match_result_free(result);
     result = NULL;
 
-    // No cleanup_params needed for stack-allocated params_ac
+    // Free the trie
+    ac_trie_free(params_ac.ac_trie);
 }
 
 /**
@@ -932,21 +953,38 @@ void test_additional_cases(void)
 
     // --- Multiple patterns (Aho-Corasick) ---
     {
-        const char *text = "foo bar baz foo bar";
-        const char *ac_patterns[] = {"foo", "bar"};
-        size_t ac_pattern_lens[] = {3, 3};
-        search_params_t params = {
-            .patterns = ac_patterns,
-            .pattern_lens = ac_pattern_lens,
+        const char *ac_multipattern_text = "foo bar baz foo qux bar";
+        const char *multi_patterns[] = {"foo", "bar"};
+        size_t multi_lens[] = {3, 3};
+        search_params_t params_multi = {
+            .patterns = multi_patterns,
+            .pattern_lens = multi_lens,
             .num_patterns = 2,
             .case_sensitive = true,
             .use_regex = false,
             .track_positions = false,
             .count_lines_mode = false,
-            .count_matches_mode = false,
+            .count_matches_mode = true,
             .compiled_regex = NULL,
-            .max_count = SIZE_MAX};
-        TEST_ASSERT(aho_corasick_search(&params, text, strlen(text), NULL) == 4, "Aho-Corasick: finds all 'foo' and 'bar'");
+            .max_count = SIZE_MAX,
+            .ac_trie = NULL // Initialize to NULL
+        };
+
+        // Build trie before using it
+        params_multi.ac_trie = ac_trie_build(&params_multi);
+        if (!params_multi.ac_trie)
+        {
+            printf("✗ FAIL: Failed to build Aho-Corasick trie in edge case test\n");
+            tests_failed++;
+        }
+        else
+        {
+            TEST_ASSERT(
+                aho_corasick_search(&params_multi, ac_multipattern_text, strlen(ac_multipattern_text), NULL) == 4,
+                "Aho-Corasick: finds all 'foo' and 'bar'");
+            // Free the trie
+            ac_trie_free(params_multi.ac_trie);
+        }
     }
 
     // --- Binary data (should not match printable pattern) ---
